package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import collections.CollectionEnumeration;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import de.renew.simulator.*;
import de.renew.event.*;

class BindingSelectionFrame implements ActionListener, ItemListener,
  Triggerable
{
  // Accesses to this field should use class synchronisation.
  private static BindingSelectionFrame instance=null;
   
  static synchronized void open(TransitionInstance transitionInstance) {
    if (instance==null) {
      instance=new BindingSelectionFrame();
    }
    instance.setTransitionInstance(transitionInstance);
  }

  static synchronized void close() {
    if (instance!=null) {
      instance.instanceClose();
    }
  }

  private final Frame frame;
  private final TextArea currentElement;
  private final List bindingList;
  private final Button fireButton;

  private Binding[] binding;
  private TransitionInstance transitionInstance;

  private boolean dirty=false;
  private boolean inSearch=false;
  private boolean wantClose=false;
  private final TriggerCollection triggers=new TriggerCollection(this);

  private BindingSelectionFrame() {
    frame=new Frame();

    currentElement=new TextArea(5,50);
    currentElement.setEditable(false);
    frame.add(currentElement,"North");

    bindingList=new List();
    bindingList.addActionListener(this);
    bindingList.addItemListener(this);
    frame.add(bindingList,"Center");

    fireButton = new Button(" Fire ");
    fireButton.addActionListener(this);

    Button updateButton = new Button(" Update ");
    updateButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	update();
      }
    });

    Button closeButton = new Button(" Close ");
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         close();
      }
    });

    Panel buttonPanel = new Panel();
    buttonPanel.add(fireButton);
    buttonPanel.add(updateButton);
    buttonPanel.add(closeButton);
    frame.add(buttonPanel,"South");
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	frame.setVisible(false);
      }
    });
    frame.pack();
  }

  private synchronized void instanceClose() {
    if (inSearch) {
      wantClose=true;
    } else {
      closeNow();
    }
  }

  private void closeNow() {
    triggers.clear();
    frame.setVisible(false);
    bindingList.removeAll();
    transitionInstance=null;
    wantClose=false;
  }
  
  public TriggerCollection triggers() {
    return triggers;
  }

  public void itemStateChanged(ItemEvent e) {
    int type=e.getStateChange();
    String text="";
    if (type==ItemEvent.SELECTED) {
	int index=bindingList.getSelectedIndex();
	if (index>=0 && index<binding.length) {
	    text=bindingList.getItem(index);
	}
    }
    currentElement.setText(text);
    currentElement.setCaretPosition(0);
  }

  public void actionPerformed(ActionEvent e) {
    int index=bindingList.getSelectedIndex();
    if (index>=0 && index<binding.length) {
      //frame.setVisible(false);
      binding[index].execute(new ConsoleTracer("Synchronously",
	"(Manually) "),true);
      update();
    } else {
      fireButton.setEnabled(false);
    }
  }

  private void setTransitionInstance(TransitionInstance transitionInstance) {
    this.transitionInstance=transitionInstance;
    wantClose=false;
    frame.setTitle(transitionInstance+"'s possible bindings");
    update();
  }

  public void firingStarted(FiringEvent fe) {
      update();
  }

  public void firingComplete(FiringEvent fe) {
      update();
  }

  /** This event is issued whenever the corresponding TransitionInstance
   *  may have become enabled or disabled. The Listener has to check
   *  enabledness itself.
   */
  public void proposeSearch(boolean mightActivate, boolean mightDeactivate) {
    update();
  }

  private void update() {
    synchronized(this) {
      if (inSearch) {
	// In a different thread there is a search prozess running.
	// Notify it about possible new bindings and forget about it.
	dirty=true;
	return;
      } else if (wantClose) {
	closeNow();
	return;
      } else {
	// Request the start of a new search process.
	dirty=false;
	inSearch=true;
      }
    }

    // When I reach this point, I have aquired the inSearch flag.
    // I am supposed to do updates until the dirty flag remains
    // false.
    while (true) {
      CollectionEnumeration bindings=
	SimulatorHelper.findAllBindings(transitionInstance,this);
      
      final int size=bindings.numberOfRemainingElements();
      binding = new Binding[size];
      // copy Bindings to Array and rebuild the AWT list.
      bindingList.removeAll();
      for (int i=0; bindings.hasMoreElements(); ++i) {
	binding[i] = (Binding)bindings.nextElement();
	bindingList.add(binding[i].toString());
      }
      String text="";
      if (size>0) {
	bindingList.select(0);
	text=bindingList.getItem(0);
      }
      currentElement.setText(text);
      currentElement.setCaretPosition(0);
      
      fireButton.setEnabled(size != 0);
      frame.setVisible(true);
           
      synchronized(this) {
	if (wantClose) {
	  // Close the frame and make sure that the search terminates.
	  closeNow();
	  dirty=false;
	}
	if (!dirty) {
	  // No more requests. Free the inSearch lock and
	  // finally exit from this method.
	  inSearch=false;
	  return;
	} else {
	  // Start another round.
	  dirty=false;
	}
      }
    }
  }
}
