package de.renew.simulator;

import collections.*;
import de.renew.unify.Impossible;

public interface EarlyExecutable extends Executable {
  // List all phases that may occur.
  public final int BLOCK=-4;
  public final int INHIBIT=-3;
  public final int INPUT=-2;
  public final int CLEAR=-1;

  // Get the phase during which this executable should execute.
  //
  // The early executables will be executed before all
  // late executables regardless of the phase. The phase
  // is local to early executables.
  int phase();

  // Get the priority of locking this executable.
  long lockPriority();

  // Lock the resources required to execute this executable.
  void lock();

  // Verify that this action is possible. Even if it fails,
  // the executable must still be unlocked, but no rollback
  // is required or allowed.
  void verify() throws Impossible;

  // Execute this executable, but do not yet unlock the
  // resources. Only called if the verify method succeeded.
  // Unlike the overwritten method from Executable,
  // this method must not throw an exception.
  void execute(Tracer tracer);

  // Undo the actions performed by the verify method.
  // Only called if the verify method succeeded.
  // Never called jointly with the execute method.
  void rollback();

  // Unlock the resources without executing this executable.
  // Unlike in previous version, this method is always called.
  void unlock();
}
