package de.renew.simulator;

import de.renew.expression.*;
import collections.*;
import de.renew.unify.*;
import de.renew.util.*;

class FlexibleArcOccurrence implements InscriptionOccurrence {
  PlaceInstance placeInstance;
  VariableMapper mapper;
  Variable variable;
  FlexibleArc arc;

  FlexibleArcOccurrence(FlexibleArc arc, VariableMapper mapper,
    NetInstance netInstance)
  {
    this.arc=arc;
    this.mapper=mapper;
    placeInstance=netInstance.getInstance(arc.place);
  }

  public CollectionEnumeration makeBinders(Searcher searcher)
    throws Impossible 
  {
    variable=new Variable(
      arc.expression.startEvaluation(mapper,searcher.recorder,
        searcher.calcChecker),
      searcher.recorder);

    if (arc.arcType==FlexibleArc.out) {
      searcher.calcChecker.addLateVariable(variable,
	searcher.recorder);
      return new EmptyEnumeration();
    } else {
      return new SingletonEnumeration(new FlexibleArcBinder(variable,
	new InputTokenCollector(searcher.reserver,placeInstance),arc));
    }
  }

  public CollectionEnumeration makeExecutables(Copier copier,
    VariableMapper copiedMapper)
  {
    switch (arc.arcType) {
    case FlexibleArc.out:
      Variable copiedVariable=(Variable)copier.copy(variable);
      return new SingletonEnumeration(
        new FlexibleOutArcExecutable(placeInstance,copiedVariable,arc));
    case FlexibleArc.in:
    case FlexibleArc.fastBoth:
      return new SingletonEnumeration(
        new FlexibleInArcExecutable(placeInstance,variable.getValue(),
	  arc));
    default:
      throw new RuntimeException("Bad arc type.");
    }
  }
}
