package de.renew.simulator;

import collections.*;
import de.renew.unify.*;
import java.util.Vector;
import java.util.Enumeration;
import java.lang.reflect.Array;
import de.renew.function.Executor;
import de.renew.util.Value;

class FlexibleInArcExecutable implements EarlyExecutable {
  PlaceInstance placeInstance;
  Object tokens;
  FlexibleArc arc;
  
  Vector removedTokens;

  FlexibleInArcExecutable(PlaceInstance placeInstance, Object tokens,
    FlexibleArc arc)
  {
    this.placeInstance=placeInstance;
    this.tokens=tokens;
    this.arc=arc;

    removedTokens=new Vector();
  }

  public long lockPriority() {
    return placeInstance.lockOrder;
  }

  public int phase() {
    return INPUT;
  }

  public void lock() {
    placeInstance.lock.lock();
  }

  private void removeSingleToken(Object token) throws Impossible {
    if (arc.forwardFunction!=null) {
      token=arc.forwardFunction.function(token);

      // The binder is supposed to check the backward function.
      // We do not expect to detect any new inconsistencies here.
    }

    placeInstance.removeToken(token);
    removedTokens.addElement(token);
  }

  public void verify() throws Impossible {
    try {
      if (tokens==null) {
	// Let's ignore this arc. Probably the null value is meant as
	// a substitute for an empty array.
      } else if (tokens.getClass().isArray()) {
	int n=Array.getLength(tokens);
	
	Class elementType=tokens.getClass().getComponentType();
	for (int i=0;i<n;i++) {
	  Object tok=Array.get(tokens,i);
	  if (elementType.isPrimitive()) {
	    tok=new Value(tok);
	  }
	  removeSingleToken(tok);
	}
      } else {
	// Unlike in the case of output arcs, we can afford to
	// reject the firing of the transition.
	throw new Impossible();
      }
    }
    catch (Exception e) {
      // Undo all reservations.
      rollback();

      throw new Impossible();
    }
  }

  public void execute(Tracer tracer) {
    for (int i=0;i<removedTokens.size();i++) {
      Object token=removedTokens.elementAt(i);
      if (arc.arcType==FlexibleArc.fastBoth) {
	if (arc.trace) {
	  tracer.trace("    Checking "+token+" in "+placeInstance);
	}
	placeInstance.insertToken(token);
      } else {
	if (arc.trace) {
	  tracer.trace("    Removing "+token+" from "+placeInstance);
	}
      }
    }
  }

  public void rollback() {
    // We have to undo the previous removals. We cannot do this
    // without notifying the observers and listeners, because
    // it was not done silently.
    for (int i=0;i<removedTokens.size();i++) {
      placeInstance.insertToken(removedTokens.elementAt(i));
    }
  }

  public void unlock() {
    placeInstance.lock.unlock();
  }
}
