package de.renew.simulator;

import java.util.Hashtable;
import de.renew.unify.Impossible;
import collections.*;
import de.renew.util.RenewObjectOutputStream;
import java.io.IOException;

public class NetInstanceImpl implements NetInstance {

  static final long serialVersionUID = -2765155215897090138L;

  private Hashtable instanceLookup;
  private Net net;
  private int netNum;

  protected NetInstanceImpl(Net net) throws Impossible {
    if (net==null) {
      throw new Impossible();
    }
    this.net=net;
    
    netNum=net.makeNetNumber();
    
    instanceLookup=new Hashtable();

    CollectionEnumeration enum=net.places.elements();
    while (enum.hasMoreElements()) {
      Place place=(Place)enum.nextElement();
      instanceLookup.put(place,place.makeInstance(this));
    }

    enum=net.transitions.elements();
    while (enum.hasMoreElements()) {
      Transition transition=(Transition)enum.nextElement();
      instanceLookup.put(transition,new TransitionInstance(this,transition));
    }
  }

  public String toString() {
    return net.toString()+"["+netNum+"]";
  }

  public String getID() {
    return String.valueOf(netNum);
  }

  // Upon creation, no transition will be checked for enabledness
  // by a searcher, because it is not contained in the
  // search queue of possibly activated transitions.
  public void createConfirmation(Tracer tracer) {
    // tracer.trace("    Confirming the creation of "+this);

    // Second, I inform all transitions of the confirmation.
    CollectionEnumeration enum=net.transitions.elements();
    while (enum.hasMoreElements()) {
      Transition transition=(Transition)enum.nextElement();
      getInstance(transition).createConfirmation();
    }    

    // Third, the places are notified, too.
    enum=net.places.elements();
    while (enum.hasMoreElements()) {
      Place place=(Place)enum.nextElement();
      getInstance(place).createConfirmation(tracer);
    }
  }

  public Object getInstance(Object netObject) {
    return instanceLookup.get(netObject);
  }
  public PlaceInstance getInstance(Place place) {
    return (PlaceInstance)instanceLookup.get(place);
  }
  public TransitionInstance getInstance(Transition transition) {
    return (TransitionInstance)instanceLookup.get(transition);
  }

  public Net getNet() { return net; }

  public CollectionEnumeration getUplinkProviders(String channel) {
    UpdatableSeq result=new LinkedList();
    
    CollectionEnumeration enum=getNet().transitions.elements();
    while (enum.hasMoreElements()) {
      Transition transition=(Transition)enum.nextElement();
      TransitionInstance instance=getInstance(transition);
      if (instance.listensToChannel(channel)) {
	result.insertFirst(instance);
      }
    }
    return result.elements();    
  }

  /**
   * Serialization method, behaves like default writeObject
   * method except using the domain trace feature, if the
   * output is a RenewObjectOutputStream.
   * @see de.renew.util.RenewObjectOutputStream
   **/
  private void writeObject(java.io.ObjectOutputStream out) throws IOException {
    RenewObjectOutputStream rOut = null;
    if (out instanceof RenewObjectOutputStream)
      rOut = (RenewObjectOutputStream)out;
    if (rOut != null) rOut.beginDomain(this);
    out.defaultWriteObject();
    if (rOut != null) rOut.endDomain(this);
  }
}

